/*
**    TP ETI5 CPE Lyon
**    Copyright (C) 2010 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SPRING_MESH_HPP_
#define _SPRING_MESH_HPP_

#include <iostream>
#include <vector>

namespace cpe
{

    class mesh;

    /** \brief square spring mesh */
    class spring_mesh
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief constructor */
        spring_mesh(const unsigned int& N=0);


        // ********************************************* //
        // ********************************************* //
        //  Dynamic
        // ********************************************* //
        // ********************************************* //

        /** \brief compute force */
        void compute_force();

        /** \brief apply force with stepsize dt */
        void apply_force(const double& dt);


        // ********************************************* //
        // ********************************************* //
        //  Init
        // ********************************************* //
        // ********************************************* //

        /** \brief init values */
        void init();


        // ********************************************* //
        // ********************************************* //
        //  attribute
        // ********************************************* //
        // ********************************************* //

        /** \brief size N (number of vertex per row: total vertices=NxN) */
        const unsigned int& size() const;

        /** \brief get the vertices */
        const std::vector<float>& get_vertex() const;
        /** \brief get the vertices */
        std::vector<float>& get_vertex();

        /** \brief get the vertices */
        const std::vector<float>& get_vertex_storage() const;
        /** \brief get the vertices */
        std::vector<float>& get_vertex_storage();

        /** \brief get structural stiffness */
        float& get_K_structural();
        /** \brief get structural stiffness */
        const float& get_K_structural() const;

        /** \brief get shear stiffness */
        float& get_K_shear();
        /** \brief get shear stiffness */
        const float& get_K_shear() const;

        /** \brief get bending stiffness */
        float& get_K_bending();
        /** \brief get bending stiffness */
        const float& get_K_bending() const;

        /** \brief get damping coeff */
        float& get_damping();
        /** \brief get damping coeff */
        const float& get_damping() const;


        // ********************************************* //
        // ********************************************* //
        //  Convertion to mesh
        // ********************************************* //
        // ********************************************* //

        /** \brief export mesh structure */
        mesh to_mesh() const;





        // ********************************************* //
        // ********************************************* //
        //  Storage vertices
        // ********************************************* //
        // ********************************************* //

        /** \brief send current vertices to storage structure */
        void store_vertices();


    private:

        /** \brief number of vertices = NxN */
        unsigned int N;

        /** \brief vertices position */
        std::vector<float> vertices;


        /** \brief special storage of vertices (for drawing purpose) */
        std::vector<float> vertices_store;



        /** \brief force container */
        std::vector<float> forces;
        /** \brief speed container */
        std::vector<float> speed;


        /** \brief structural stiffness */
        float K_structural;
        /** \brief shear stiffness */
        float K_shear;
        /** \brief bending stiffness */
        float K_bending;

        /** \brief damping coefficient */
        float damping;


    };
}

#endif
